@@ -7,14 +7,15 @@ import numpy as np
 from astropy import units as u
 from astropy.modeling import Model, models, fitting
 from astropy.nddata import NDData, VarianceUncertainty
+from numpy import ndarray
 from scipy.integrate import trapezoid
 from scipy.interpolate import RectBivariateSpline
 from specutils import Spectrum1D
 
-from specreduce.core import SpecreduceOperation
+from specreduce.core import SpecreduceOperation, ImageLike, MaskingOption
 from specreduce.tracing import Trace, FlatTrace
 
-__all__ = ['BoxcarExtract', 'HorneExtract', 'OptimalExtract']
+__all__ = ["BoxcarExtract", "HorneExtract", "OptimalExtract"]
 
 
 def _get_boxcar_weights(center, hwidth, npix):
@@ -50,12 +51,12 @@ def _get_boxcar_weights(center, hwidth, npix):
         # (negative widths should be avoided by earlier logic!)
         return weights
 
-    if center-hwidth > npix-0.5 or center+hwidth < -0.5:
+    if center - hwidth > npix - 0.5 or center + hwidth < -0.5:
         # entire window is out-of-bounds
         return weights
 
-    lower_edge = max(-0.5, center-hwidth)  # where -0.5 is lower bound of the image
-    upper_edge = min(center+hwidth, npix-0.5)  # where npix-0.5 is upper bound of the image
+    lower_edge = max(-0.5, center - hwidth)  # where -0.5 is lower bound of the image
+    upper_edge = min(center + hwidth, npix - 0.5)  # where npix-0.5 is upper bound of the image
 
     # let's avoid recomputing the round repeatedly
     int_round_lower_edge = int(round(lower_edge))
@@ -66,21 +67,24 @@ def _get_boxcar_weights(center, hwidth, npix):
     # the upper bound doesn't have the +1 because array slicing is inclusive on the lower index and
     # exclusive on the upper-index
     # NOTE: round(-0.5) == 0, which is helpful here for the case where lower_edge == -0.5
-    weights[int_round_lower_edge+1:int_round_upper_edge] = 1
+    weights[int_round_lower_edge + 1 : int_round_upper_edge] = 1
 
     # handle edge pixels (for cases where an edge pixel is fully-weighted, this will set it again,
     # but should still compute a weight of 1.  By using N:N+1, we avoid index errors if the edge
     # is outside the image bounds.  But we do need to avoid negative indices which would count
     # from the end of the array.
     if int_round_lower_edge >= 0:
-        weights[int_round_lower_edge:int_round_lower_edge+1] = round(lower_edge) + 0.5 - lower_edge
-    weights[int_round_upper_edge:int_round_upper_edge+1] = upper_edge - (round(upper_edge) - 0.5)
+        weights[int_round_lower_edge : int_round_lower_edge + 1] = (
+            round(lower_edge) + 0.5 - lower_edge
+        )
+    weights[int_round_upper_edge : int_round_upper_edge + 1] = upper_edge - (
+        round(upper_edge) - 0.5
+    )
 
     return weights
 
 
 def _ap_weight_image(trace, width, disp_axis, crossdisp_axis, image_shape):
-
     """
     Create a weight image that defines the desired extraction aperture.
 
@@ -113,6 +117,10 @@ def _ap_weight_image(trace, width, disp_axis, crossdisp_axis, image_shape):
     for i in range(image_shape[disp_axis]):
         # TODO trace must handle transposed data (disp_axis == 0)
         # pass trace.trace.data[i] to avoid any mask if part of the regions is out-of-bounds
+
+        # ArrayTrace can have nonfinite or masked data in trace, and this will fail,
+        # so figure out how to handle that...
+
         wimage[:, i] = _get_boxcar_weights(trace.trace.data[i], hwidth, image_sizes)
 
     return wimage
@@ -121,7 +129,7 @@ def _ap_weight_image(trace, width, disp_axis, crossdisp_axis, image_shape):
 @dataclass
 class BoxcarExtract(SpecreduceOperation):
     """
-    Does a standard boxcar extraction.
+    Standard boxcar extraction along a trace.
 
     Example: ::
 
@@ -132,89 +140,125 @@ class BoxcarExtract(SpecreduceOperation):
 
     Parameters
     ----------
-    image : `~astropy.nddata.NDData`-like or array-like, required
+    image
         image with 2-D spectral image data
-    trace_object : Trace, required
+    trace_object
         trace object
-    width : float, optional
+    width
         width of extraction aperture in pixels
-    disp_axis : int, optional
+    disp_axis
         dispersion axis
-    crossdisp_axis : int, optional
+    crossdisp_axis
         cross-dispersion axis
+    mask_treatment
+        Specifies how to handle masked or non-finite values in the input image.
+        The accepted values are:
+
+        - ``apply``: The image remains unchanged, and any existing mask is combined\
+            with a mask derived from non-finite values.
+        - ``ignore``: The image remains unchanged, and any existing mask is dropped.
+        - ``propagate``: The image remains unchanged, and any masked or non-finite pixel\
+            causes the mask to extend across the entire cross-dispersion axis.
+        - ``zero_fill``: Pixels that are either masked or non-finite are replaced with 0.0,\
+            and the mask is dropped.
+        - ``nan_fill``:  Pixels that are either masked or non-finite are replaced with nan,\
+            and the mask is dropped.
+        - ``apply_mask_only``: The  image and mask are left unmodified.
+        - ``apply_nan_only``: The  image is left unmodified, the old mask is dropped, and a\
+            new mask is created based on non-finite values.
 
     Returns
     -------
     spec : `~specutils.Spectrum1D`
         The extracted 1d spectrum expressed in DN and pixel units
     """
-    image: NDData
+
+    image: ImageLike
     trace_object: Trace
     width: float = 5
     disp_axis: int = 1
     crossdisp_axis: int = 0
     # TODO: should disp_axis and crossdisp_axis be defined in the Trace object?
+    mask_treatment: MaskingOption = "apply"
+    _valid_mask_treatment_methods = (
+        "apply",
+        "ignore",
+        "propagate",
+        "zero_fill",
+        "nan_fill",
+        "apply_mask_only",
+        "apply_nan_only",
+    )
 
     @property
     def spectrum(self):
         return self.__call__()
 
-    def __call__(self, image=None, trace_object=None, width=None,
-                 disp_axis=None, crossdisp_axis=None):
+    def __call__(
+        self,
+        image: ImageLike | None = None,
+        trace: Trace | None = None,
+        width: float | None = None,
+        disp_axis: int | None = None,
+        crossdisp_axis: int | None = None,
+    ) -> Spectrum1D:
         """
         Extract the 1D spectrum using the boxcar method.
 
         Parameters
         ----------
-        image : `~astropy.nddata.NDData`-like or array-like, required
-            image with 2-D spectral image data
-        trace_object : Trace, required
-            trace object
-        width : float, optional
-            width of extraction aperture in pixels [default: 5]
-        disp_axis : int, optional
-            dispersion axis [default: 1]
-        crossdisp_axis : int, optional
-            cross-dispersion axis [default: 0]
-
+        image
+            The image with 2-D spectral image data
+        trace
+            The trace object
+        width
+            The width of extraction aperture in pixels
+        disp_axis
+            The dispersion axis
+        crossdisp_axis
+            The cross-dispersion axis
 
         Returns
         -------
-        spec : `~specutils.Spectrum1D`
+        spec
             The extracted 1d spectrum with flux expressed in the same
             units as the input image, or u.DN, and pixel units
         """
         image = image if image is not None else self.image
-        trace_object = trace_object if trace_object is not None else self.trace_object
-        width = width if width is not None else self.width
-        disp_axis = disp_axis if disp_axis is not None else self.disp_axis
-        crossdisp_axis = crossdisp_axis if crossdisp_axis is not None else self.crossdisp_axis
+        trace = trace or self.trace_object
+        width = width or self.width
+        disp_axis = disp_axis or self.disp_axis
+        cdisp_axis = crossdisp_axis or self.crossdisp_axis
 
-        # handle image processing based on its type
-        self.image = self._parse_image(image)
+        if width <= 0:
+            raise ValueError("The window width must be positive")
 
-        # TODO: this check can be removed if/when implemented as a check in FlatTrace
-        if isinstance(trace_object, FlatTrace):
-            if trace_object.trace_pos < 1:
-                raise ValueError('trace_object.trace_pos must be >= 1')
+        self.image = self._parse_image(
+            image, disp_axis=disp_axis, mask_treatment=self.mask_treatment
+        )
 
-        if width < 0:
-            raise ValueError("width must be positive")
+        # Spectrum extraction
+        # ===================
+        # Assign no weight to non-finite pixels outside the window. Non-finite pixels inside
+        # the window will be propagated to the sum if mask treatment is either ``ignore`` or
+        # ``propagate`` or excluded if the chosen mask treatment option is ``apply``. In the
+        # latter case, the flux is calculated as the average of the non-masked pixels inside
+        # the window multiplied by the window width.
+        window_weights = _ap_weight_image(trace, width, disp_axis, cdisp_axis, self.image.shape)
 
-        # weight image to use for extraction
-        wimg = _ap_weight_image(
-            trace_object,
-            width,
-            disp_axis,
-            crossdisp_axis,
-            self.image.shape)
+        if self.mask_treatment == "apply":
+            image_cleaned = np.where(~self.image.mask, self.image.data * window_weights, 0.0)
+            weights = np.where(~self.image.mask, window_weights, 0.0)
+            spectrum = (
+                image_cleaned.sum(axis=cdisp_axis)
+                / weights.sum(axis=cdisp_axis)
+                * window_weights.sum(axis=cdisp_axis)
+            )
+        else:
+            image_windowed = np.where(window_weights, self.image.data * window_weights, 0.0)
+            spectrum = np.sum(image_windowed, axis=cdisp_axis)
 
-        # extract, assigning no weight to non-finite pixels outside the window
-        # (non-finite pixels inside the window will still make it into the sum)
-        image_windowed = np.where(wimg, self.image.data*wimg, 0)
-        ext1d = np.sum(image_windowed, axis=crossdisp_axis)
-        return Spectrum1D(ext1d * self.image.unit,
-                          spectral_axis=self.image.spectral_axis)
+        return Spectrum1D(spectrum * self.image.unit, spectral_axis=self.image.spectral_axis)
 
 
 @dataclass
@@ -227,8 +271,9 @@ class HorneExtract(SpecreduceOperation):
     extraction - by default, a 1D gaussian is fit and as a uniform profile
     across the spectrum. Alternativley, the ``self profile`` option may be
     chosen - when this option is chosen, the spatial profile will be sampled
-    at various locations (set by <>) and interpolated between to produce a
-    smoothly varying spatial profile across the spectrum.
+    (using a default of 10 sample bins, but can be modified with
+    ``spatial_profile``) and interpolated between to produce a smoothly varying
+    spatial profile across the spectrum.
 
     If using the Gaussian option for the spatial profile, a background profile
     may be fit (but not subtracted) simultaneously to the data. By default,
@@ -293,10 +338,11 @@ class HorneExtract(SpecreduceOperation):
         fluxes are interpreted in DN. [default: None]
 
     """
+
     image: NDData
     trace_object: Trace
     bkgrd_prof: Model = field(default=models.Polynomial1D(2))
-    spatial_profile: str = 'gaussian'  # can actually be str, dict
+    spatial_profile: str | dict = "gaussian"
     variance: np.ndarray = field(default=None)
     mask: np.ndarray = field(default=None)
     unit: np.ndarray = field(default=None)
@@ -308,8 +354,7 @@ class HorneExtract(SpecreduceOperation):
     def spectrum(self):
         return self.__call__()
 
-    def _parse_image(self, image,
-                     variance=None, mask=None, unit=None, disp_axis=1):
+    def _parse_image(self, image, variance=None, mask=None, unit=None, disp_axis=1):
         """
         Convert all accepted image types to a consistently formatted
         Spectrum1D object.
@@ -357,7 +402,7 @@ class HorneExtract(SpecreduceOperation):
         # mask is set as None when not specified upon creating a Spectrum1D
         # object, so we must check whether it is absent *and* whether it's
         # present but set as None
-        if getattr(image, 'mask', None) is not None:
+        if getattr(image, "mask", None) is not None:
             mask = image.mask
         elif mask is not None:
             pass
@@ -366,37 +411,41 @@ class HorneExtract(SpecreduceOperation):
             mask = np.zeros_like(img)
 
         if img.shape != mask.shape:
-            raise ValueError('image and mask shapes must match.')
+            raise ValueError("image and mask shapes must match.")
 
         # Process uncertainties, converting to variances when able and throwing
         # an error when uncertainties are missing or less easily converted
-        if (hasattr(image, 'uncertainty')
-                and image.uncertainty is not None):
-            if image.uncertainty.uncertainty_type == 'var':
+        if hasattr(image, "uncertainty") and image.uncertainty is not None:
+            if image.uncertainty.uncertainty_type == "var":
                 variance = image.uncertainty.array
-            elif image.uncertainty.uncertainty_type == 'std':
-                warnings.warn("image NDData object's uncertainty "
-                              "interpreted as standard deviation. if "
-                              "incorrect, use VarianceUncertainty when "
-                              "assigning image object's uncertainty.")
+            elif image.uncertainty.uncertainty_type == "std":
+                warnings.warn(
+                    "image NDData object's uncertainty "
+                    "interpreted as standard deviation. if "
+                    "incorrect, use VarianceUncertainty when "
+                    "assigning image object's uncertainty."
+                )
                 variance = image.uncertainty.array**2
-            elif image.uncertainty.uncertainty_type == 'ivar':
+            elif image.uncertainty.uncertainty_type == "ivar":
                 variance = 1 / image.uncertainty.array
             else:
                 # other options are InverseUncertainty and UnknownUncertainty
-                raise ValueError("image NDData object has unexpected "
-                                 "uncertainty type. instead, try "
-                                 "VarianceUncertainty or StdDevUncertainty.")
-        elif (hasattr(image, 'uncertainty')
-              and image.uncertainty is None):
+                raise ValueError(
+                    "image NDData object has unexpected "
+                    "uncertainty type. instead, try "
+                    "VarianceUncertainty or StdDevUncertainty."
+                )
+        elif hasattr(image, "uncertainty") and image.uncertainty is None:
             # ignore variance arg to focus on updating NDData object
-            raise ValueError('image NDData object lacks uncertainty')
+            raise ValueError("image NDData object lacks uncertainty")
         else:
             if variance is None:
-                raise ValueError("if image is a numpy or Quantity array, a "
-                                 "variance must be specified. consider "
-                                 "wrapping it into one object by instead "
-                                 "passing an NDData image.")
+                raise ValueError(
+                    "if image is a numpy or Quantity array, a "
+                    "variance must be specified. consider "
+                    "wrapping it into one object by instead "
+                    "passing an NDData image."
+                )
             elif image.shape != variance.shape:
                 raise ValueError("image and variance shapes must match")
 
@@ -414,41 +463,32 @@ class HorneExtract(SpecreduceOperation):
 
         variance = VarianceUncertainty(variance)
 
-        unit = getattr(image, 'unit',
-                       u.Unit(unit) if unit is not None else u.Unit('DN'))
+        unit = getattr(image, "unit", u.Unit(unit) if unit is not None else u.Unit("DN"))
 
-        spectral_axis = getattr(image, 'spectral_axis',
-                                np.arange(img.shape[disp_axis]) * u.pix)
+        spectral_axis = getattr(image, "spectral_axis", np.arange(img.shape[disp_axis]) * u.pix)
 
-        return Spectrum1D(img * unit, spectral_axis=spectral_axis,
-                          uncertainty=variance, mask=mask)
+        return Spectrum1D(img * unit, spectral_axis=spectral_axis, uncertainty=variance, mask=mask)
 
-    def _fit_gaussian_spatial_profile(self, img, disp_axis, crossdisp_axis,
-                                      or_mask, bkgrd_prof):
+    def _fit_gaussian_spatial_profile(
+        self, img: ndarray, disp_axis: int, crossdisp_axis: int, or_mask: ndarray, bkgrd_prof: Model
+    ):
+        """Fit a 1D Gaussian spatial profile to spectrum in `img`.
 
+        Fits an 1D Gaussian profile to spectrum in `img`. Takes the weighted mean
+        of  ``img`` along the cross-dispersion axis all ignoring masked pixels
+        (i.e, takes the mean of each row for a horizontal trace). A Background model
+        (optional) is fit simultaneously. Returns an `astropy.model.Gaussian1D` (or
+        compound model, if `bkgrd_prof` is supplied) fit to data.
         """
-            Fits an 1D Gaussian profile to spectrum in `img`. Takes the mean
-            of  ``img`` along the cross-dispersion axis (i.e, takes the mean of
-            each row for a horizontal trace). Any columns with non-finite values
-            are omitted from the fit. Background model (optional) is fit
-            simultaneously. Returns an `astropy.model.Gaussian1D` (or compound
-            model, if `bkgrd_prof` is supplied) fit to data.
-        """
-
-        # co-add signal in each image row
         nrows = img.shape[crossdisp_axis]
-        ncols = img.shape[disp_axis]
         xd_pixels = np.arange(nrows)
 
-        # for now, mask row with any non-finite value.
-        row_mask = np.logical_or.reduce(or_mask, axis=disp_axis)
-        coadd = np.ma.masked_array(np.sum(img, axis=disp_axis) / ncols,
-                                   mask=row_mask)
+        # co-add signal in each image row, ignore masked pixels
+        coadd = np.ma.masked_array(img, mask=or_mask).mean(disp_axis)
 
         # use the sum of brightest row as an inital guess for Gaussian amplitude,
         # the the location of the brightest row as an initial guess for the mean
-        gauss_prof = models.Gaussian1D(amplitude=coadd.max(),
-                                       mean=coadd.argmax(), stddev=2)
+        gauss_prof = models.Gaussian1D(amplitude=coadd.max(), mean=coadd.argmax(), stddev=2)
 
         # Fit extraction kernel (Gaussian + background model) to coadded rows
         # with combined model (must exclude masked indices manually;
@@ -457,70 +497,89 @@ class HorneExtract(SpecreduceOperation):
             ext_prof = gauss_prof + bkgrd_prof
         else:
             # add a trivial constant model so attribute names are the same
-            ext_prof = gauss_prof + models.Const1D(0, fixed={'amplitude': True})
-
-        fitter = fitting.LevMarLSQFitter()
-        fit_ext_kernel = fitter(ext_prof, xd_pixels[~row_mask], coadd[~row_mask])
+            ext_prof = gauss_prof + models.Const1D(0, fixed={"amplitude": True})
 
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore")
+            fitter = fitting.LMLSQFitter()
+            fit_ext_kernel = fitter(ext_prof, xd_pixels[~coadd.mask], coadd.compressed())
         return fit_ext_kernel
 
-    def _fit_self_spatial_profile(self, img, disp_axis, crossdisp_axis, or_mask,
-                                  n_bins_interpolated_profile, kx, ky):
-
+    def _fit_spatial_profile(
+        self,
+        img: ndarray,
+        disp_axis: int,
+        crossdisp_axis: int,
+        mask: ndarray,
+        n_bins: int,
+        kx: int,
+        ky: int,
+    ) -> RectBivariateSpline:
         """
-            Fit a spatial profile to spectrum by sampling the median profile in
-            bins (number of which set be `n_bins_interpolated_profile` along the
-            dispersion direction, and interpolating between
-            samples. Columns (assuming horizontal trace) with any non-finite
-            values will be omitted from the fit. Returns an interpolator object
-            (RectBivariateSpline) that can be evaluated at any x,y.
+        Fit a spatial profile by sampling the median profile along the dispersion direction.
+
+        This method extracts the spatial profile from an input spectrum by binning
+        the data along the dispersion axis. It calculates the median profile for each bin,
+        normalizes it, and then interpolates between these profiles to create a smooth
+        2D representation of the spatial profile. The resulting interpolator object can be
+        used to evaluate the spatial profile at any coordinate within the bounds of the data.
+
+        Parameters
+        ----------
+        img
+            The 2D array of spectral data to process.
+        disp_axis
+            The image axis corresponding to the dispersion direction.
+        crossdisp_axis
+            The image axis corresponding to the cross-dispersion direction.
+        mask
+            A boolean mask array with the same shape as the image. Values of ``True``
+            in the mask indicate invalid data points to be ignored during computation.
+        n_bins
+            The number of bins to use along the dispersion axis for sampling
+            the median spatial profile.
+        kx
+            The degree of the spline along the dispersion axis.
+        ky
+            The degree of the spline along the cross-dispersion axis.
+
+        Returns
+        -------
+        RectBivariateSpline
+            Interpolator object that provides a smoothed 2D spatial profile.
         """
+        img = np.where(~mask, img, np.nan)
+        nrows = img.shape[crossdisp_axis]
+        ncols = img.shape[disp_axis]
+        samples = np.zeros((n_bins, nrows))
 
-        # boundaries of bins for sampling profile.
-        sample_locs = np.linspace(0, img.shape[disp_axis]-1,
-                                  n_bins_interpolated_profile+1, dtype=int)
-        # centers of these bins, roughly
-        bin_centers = [(sample_locs[i]+sample_locs[i+1]) // 2 for i in
-                       range(len(sample_locs) - 1)]
+        sample_locs = np.linspace(0, ncols - 1, n_bins + 1, dtype=int)
+        bin_centers = [
+            (sample_locs[i] + sample_locs[i + 1]) // 2 for i in range(len(sample_locs) - 1)
+        ]
 
-        # for now, since fitting isn't enabled for arrays with any nans
-        # mask out the columns with any non-finite values to make sure
-        # these don't contribute to the fit profile
-        col_mask = np.logical_or.reduce(or_mask, axis=crossdisp_axis)
+        for i in range(n_bins):
+            bin_median = np.nanmedian(img[:, sample_locs[i] : sample_locs[i + 1]], axis=disp_axis)
+            samples[i, :] = bin_median / bin_median.sum()
 
-        # make a full mask for the image based on which cols have nans
-        img_col_mask = np.tile(col_mask, (img.shape[0], 1))
+        return RectBivariateSpline(x=bin_centers, y=np.arange(nrows), z=samples, kx=kx, ky=ky)
 
-        # need to make a new masked array since this mask is different
-        # omit all columns with nans from contributing to the fit
-        new_masked_arr = np.ma.array(img.data.copy(), mask=img_col_mask)
-
-        # sample at these locations, normalize to area so this just reflects the
-        # shape of the spectrum any shifts in center location should be corrected
-        # by _align_along_trace (this should be addressed later with a better way
-        # to flatten the trace, because trace can be wiggly even if 'flat'...)
-        samples = []
-        for i in range(n_bins_interpolated_profile):
-            slicee = new_masked_arr[:, sample_locs[i]:sample_locs[i+1]]
-            bin_median = np.ma.median(slicee, axis=disp_axis)
-            bin_median_sum = np.ma.sum(bin_median)
-            samples.append(bin_median / bin_median_sum)
-
-        interp_2d = RectBivariateSpline(x=bin_centers,
-                                        y=np.arange(img.shape[crossdisp_axis]),
-                                        z=samples, kx=kx, ky=ky)
-
-        return interp_2d
-
-    def __call__(self, image=None, trace_object=None,
-                 disp_axis=None, crossdisp_axis=None,
-                 bkgrd_prof=None, spatial_profile=None,
-                 n_bins_interpolated_profile=None,
-                 interp_degree_interpolated_profile=None,
-                 variance=None, mask=None, unit=None):
+    def __call__(
+        self,
+        image=None,
+        trace_object=None,
+        disp_axis=None,
+        crossdisp_axis=None,
+        bkgrd_prof=None,
+        spatial_profile=None,
+        n_bins_interpolated_profile=None,
+        interp_degree_interpolated_profile=None,
+        variance=None,
+        mask=None,
+        unit=None,
+    ):
         """
-        Run the Horne calculation on a region of an image and extract a
-        1D spectrum.
+        Run the Horne calculation on a region of an image and extract a 1D spectrum.
 
         Parameters
         ----------
@@ -587,113 +646,59 @@ class HorneExtract(SpecreduceOperation):
         disp_axis = disp_axis if disp_axis is not None else self.disp_axis
         crossdisp_axis = crossdisp_axis if crossdisp_axis is not None else self.crossdisp_axis
         bkgrd_prof = bkgrd_prof if bkgrd_prof is not None else self.bkgrd_prof
-        spatial_profile = (spatial_profile if spatial_profile is not None else
-                           self.spatial_profile)
+        profile = spatial_profile if spatial_profile is not None else self.spatial_profile
         variance = variance if variance is not None else self.variance
         mask = mask if mask is not None else self.mask
         unit = unit if unit is not None else self.unit
 
-        # figure out what 'spatial_profile' was provided
-        # put this parsing into another method at some point, its a lot..
-        interp_degree_interpolated_profile = None
-        n_bins_interpolated_profile = None
-        spatial_profile_choices = ('gaussian', 'interpolated_profile')
+        profile_choices = ("gaussian", "interpolated_profile")
 
-        if isinstance(spatial_profile, str):
-            spatial_profile = spatial_profile.lower()
-            if spatial_profile not in spatial_profile_choices:
-                raise ValueError("spatial_profile must be one of"
-                                 f"{', '.join(spatial_profile_choices)}")
-            if spatial_profile == 'interpolated_profile':  # use defaults
-                bkgrd_prof = None
-                n_bins_interpolated_profile = 10
-                interp_degree_interpolated_profile = 1
-        elif isinstance(spatial_profile, dict):
-            # first, figure out what type of profile is indicated
-            # right now, the only type that should use a dictionary is 'interpolated_profile'
-            # but this may be extended in the future hence the 'name' key. also,
-            # the gaussian option could be supplied as a single-key dict
+        if not isinstance(profile, (str, dict)):
+            raise ValueError("spatial_profile must be a string or dictionary.")
+        if isinstance(profile, str):
+            profile = dict(name=profile)
 
-            # will raise key error if not present, and also fail on .lower if not
-            # a string - think this is informative enough
-            spatial_profile_type = spatial_profile['name'].lower()
+        profile_type = profile["name"].lower()
+        if profile_type not in profile_choices:
+            raise ValueError("spatial_profile must be one of" f"{', '.join(profile_choices)}")
 
-            if spatial_profile_type not in spatial_profile_choices:
-                raise ValueError("spatial_profile must be one of"
-                                 f"{', '.join(spatial_profile_choices)}")
-            if spatial_profile_type == 'gaussian':
-                spatial_profile = 'gaussian'
-            else:
-                if 'n_bins_interpolated_profile' in spatial_profile.keys():
-                    n_bins_interpolated_profile = \
-                        spatial_profile['n_bins_interpolated_profile']
-                else:  # use default
-                    n_bins_interpolated_profile = 10
-                if 'interp_degree_interpolated_profile' in spatial_profile.keys():
-                    interp_degree_interpolated_profile = \
-                        spatial_profile['interp_degree_interpolated_profile']
-                else:  # use default
-                    interp_degree_interpolated_profile = 1
-            spatial_profile = spatial_profile_type
-            bkgrd_prof = None
-        else:
-            raise ValueError('``spatial_profile`` must either be string or dictionary.')
+        n_bins_interpolated_profile = profile.get("n_bins_interpolated_profile", 10)
+        interp_degree_interpolated_profile = profile.get("interp_degree_interpolated_profile", 1)
 
-        # parse image and replace optional arguments with updated values
         self.image = self._parse_image(image, variance, mask, unit, disp_axis)
-        variance = self.image.uncertainty.array
-        mask = self.image.mask
+        variance = self.image.uncertainty.represent_as(VarianceUncertainty).array
+        mask = self.image.mask.astype(bool) | (~np.isfinite(self.image.data))
         unit = self.image.unit
+        img = self.image.data
 
-        img = np.ma.masked_array(self.image.data, mask=mask)
-
-        # create separate mask including any previously uncaught non-finite
-        # values for purposes of calculating fit
-        or_mask = np.logical_or(img.mask,
-                                ~np.isfinite(self.image.data))
+        ncross = img.shape[crossdisp_axis]
+        ndisp = img.shape[disp_axis]
 
         # If the trace is not flat, shift the rows in each column
         # so the image is aligned along the trace:
         if not isinstance(trace_object, FlatTrace):
             img = _align_along_trace(
-                img,
-                trace_object.trace,
-                disp_axis=disp_axis,
-                crossdisp_axis=crossdisp_axis)
+                img, trace_object.trace, disp_axis=disp_axis, crossdisp_axis=crossdisp_axis
+            )
 
-        if self.spatial_profile == 'gaussian':
-
-            # fit profile to average (mean) profile along crossdisp axis
-            fit_ext_kernel = self._fit_gaussian_spatial_profile(img,
-                                                                disp_axis,
-                                                                crossdisp_axis,
-                                                                or_mask,
-                                                                bkgrd_prof)
-
-            # this is just creating an array of the trace to shift the mean
-            # when iterating over each wavelength. this needs to be fixed in the
-            # future to actually account for the trace shape in a non-flat trace
-            # (or possibly omitted all togehter as it might be redundant if
-            # _align_along_trace is correcting this already)
+        if profile_type == "gaussian":
+            fit_ext_kernel = self._fit_gaussian_spatial_profile(
+                img, disp_axis, crossdisp_axis, mask, bkgrd_prof
+            )
             if isinstance(trace_object, FlatTrace):
-                mean_init_guess = trace_object.trace
+                mean_cross_pix = trace_object.trace
             else:
-                mean_init_guess = np.broadcast_to(
-                    img.shape[crossdisp_axis] // 2, img.shape[disp_axis]
-                )
-
+                mean_cross_pix = np.broadcast_to(ncross // 2, ndisp)
         else:  # interpolated_profile
             # for now, bkgrd_prof must be None because a compound model can't
             # be created with a interpolator + model. i think there is a way
             # around this, but will follow up later
             if bkgrd_prof is not None:
-                raise ValueError('When `spatial_profile`is `interpolated_profile`,'
-                                 '`bkgrd_prof` must be None. Background should'
-                                 ' be fit and subtracted from `img` beforehand.')
-            # make sure n_bins doesnt exceed the number of (for now) finite
-            # columns. update this when masking is fixed.
-            n_finite_cols = np.logical_or.reduce(or_mask, axis=crossdisp_axis)
-            n_finite_cols = np.count_nonzero(n_finite_cols.astype(int) == 0)
+                raise ValueError(
+                    "When `spatial_profile`is `interpolated_profile`,"
+                    "`bkgrd_prof` must be None. Background should"
+                    " be fit and subtracted from `img` beforehand."
+                )
 
             # determine interpolation degree from input and make tuple if int
             # this can also be moved to another method to parse the input
@@ -701,91 +706,51 @@ class HorneExtract(SpecreduceOperation):
             if isinstance(interp_degree_interpolated_profile, int):
                 kx = ky = interp_degree_interpolated_profile
             else:  # if input is tuple of ints
-
                 if not isinstance(interp_degree_interpolated_profile, tuple):
-                    raise ValueError("``interp_degree_interpolated_profile`` must be ",
-                                     "an integer or tuple of integers.")
+                    raise ValueError(
+                        "``interp_degree_interpolated_profile`` must be ",
+                        "an integer or tuple of integers.",
+                    )
                 if not all(isinstance(x, int) for x in interp_degree_interpolated_profile):
-                    raise ValueError("``interp_degree_interpolated_profile`` must be ",
-                                     "an integer or tuple of integers.")
-
+                    raise ValueError(
+                        "``interp_degree_interpolated_profile`` must be ",
+                        "an integer or tuple of integers.",
+                    )
                 kx, ky = interp_degree_interpolated_profile
 
-            if n_bins_interpolated_profile >= n_finite_cols:
-
-                raise ValueError(f'`n_bins_interpolated_profile` ({n_bins_interpolated_profile}) '
-                                 'must be less than the number of fully-finite '
-                                 f'wavelength columns ({n_finite_cols}).')
-
-            interp_spatial_prof = self._fit_self_spatial_profile(img, disp_axis,
-                                                                 crossdisp_axis,
-                                                                 or_mask,
-                                                                 n_bins_interpolated_profile,
-                                                                 kx, ky)
+            interp_spatial_prof = self._fit_spatial_profile(
+                img, disp_axis, crossdisp_axis, mask, n_bins_interpolated_profile, kx, ky
+            )
 
             # add private attribute to save fit profile. should this be public?
             self._interp_spatial_prof = interp_spatial_prof
 
-        col_mask = np.logical_or.reduce(or_mask, axis=crossdisp_axis)
-        nonf_col = [np.nan] * img.shape[crossdisp_axis]
+        xd_pixels = np.arange(ncross)
+        kernel_vals = np.zeros(img.shape)
+        norms = np.full(ndisp, np.nan)
+        valid = ~mask
 
-        # array of 'x' values for each wavelength for extraction
-        nrows = img.shape[crossdisp_axis]
-        xd_pixels = np.arange(nrows)
+        if profile_type == "gaussian":
+            norms[:] = fit_ext_kernel.amplitude_0 * fit_ext_kernel.stddev_0 * np.sqrt(2 * np.pi)
 
-        kernel_vals = []
-        norms = []
-        for col_pix in range(img.shape[disp_axis]):
-
-            # for now, skip columns with any non-finite values
-            # NOTE: fit and other kernel operations should support masking again
-            # once a fix is in for renormalizing columns with non-finite values
-            if col_mask[col_pix]:
-                kernel_vals.append(nonf_col)
-                norms.append(np.nan)
+        for idisp in range(ndisp):
+            if not np.any(valid[:, idisp]):
                 continue
-
-            if self.spatial_profile == 'gaussian':
-
-                # set compound model's mean to column's matching trace value
-                # again, this is probably not necessary
-                if bkgrd_prof is not None:  # attr names will be diff. if not compound
-                    fit_ext_kernel.mean_0 = mean_init_guess[col_pix]
-                else:
-                    fit_ext_kernel.mean = mean_init_guess[col_pix]
-
-                # evaluate fit model (with shifted mean, based on trace)
+            if profile_type == "gaussian":
+                fit_ext_kernel.mean_0 = mean_cross_pix[idisp]
                 fitted_col = fit_ext_kernel(xd_pixels)
+                kernel_vals[:, idisp] = fitted_col
+            else:
+                fitted_col = interp_spatial_prof(idisp, xd_pixels)
+                kernel_vals[:, idisp] = fitted_col
+                norms[idisp] = trapezoid(fitted_col, dx=1)[0]
 
-                # save result and normalization
-                # this doesn't need to be in this loop, address later
-                kernel_vals.append(fitted_col)
+        with np.errstate(divide="ignore", invalid="ignore"):
+            num = np.sum(np.where(valid, img * kernel_vals / variance, 0.0), axis=crossdisp_axis)
+            den = np.sum(np.where(valid, kernel_vals**2 / variance, 0.0), axis=crossdisp_axis)
+            extraction = (num / den) * norms
 
-                norms.append(fit_ext_kernel.amplitude_0
-                             * fit_ext_kernel.stddev_0 * np.sqrt(2*np.pi))
-
-            else:  # interpolated_profile
-                fitted_col = interp_spatial_prof(col_pix, xd_pixels)
-                kernel_vals.append(fitted_col)
-                norms.append(trapezoid(fitted_col, dx=1)[0])
-
-        # transform fit-specific information
-        kernel_vals = np.vstack(kernel_vals).T
-        norms = np.array(norms)
-
-        # calculate kernel normalization
-        g_x = np.sum(kernel_vals**2 / variance, axis=crossdisp_axis)
-
-        # sum by column weights
-        weighted_img = np.divide(img * kernel_vals, variance)
-        result = np.sum(weighted_img, axis=crossdisp_axis) / g_x
-
-        # multiply kernel normalization into the extracted signal
-        extraction = result * norms
-
-        # convert the extraction to a Spectrum1D object
-        return Spectrum1D(extraction * unit,
-                          spectral_axis=self.image.spectral_axis)
+        return Spectrum1D(extraction * unit, spectral_axis=self.image.spectral_axis)
 
 
 def _align_along_trace(img, trace_array, disp_axis=1, crossdisp_axis=0):
@@ -821,5 +786,6 @@ class OptimalExtract(HorneExtract):
     """
     An alias for `HorneExtract`.
     """
+
     __doc__ += HorneExtract.__doc__
     pass
